@@ -34,21 +34,44 @@
 #include <backends/imgui_impl_glfw.h>
 #include <backends/imgui_impl_opengl3.h>
 
-// All headers not in the root directory require this
-#define GLM_ENABLE_EXPERIMENTAL 1
-#include <glm/glm.hpp>
-#include <glm/gtc/matrix_transform.hpp>
-#include <glm/gtc/matrix_inverse.hpp>
-#include <glm/gtc/type_ptr.hpp>
-#include <glm/gtx/quaternion.hpp>
-
 #define STB_IMAGE_IMPLEMENTATION
 #include "stb_image.h"
 
 #include <fastgltf/core.hpp>
 #include <fastgltf/types.hpp>
 #include <fastgltf/tools.hpp>
-#include <fastgltf/glm_element_traits.hpp>
+
+// It's simpler here to just declare the functions as part of the fastgltf::math namespace.
+namespace fastgltf::math {
+	/** Creates a right-handed view matrix */
+	[[nodiscard]] auto lookAtRH(const fvec3& eye, const fvec3& center, const fvec3& up) noexcept {
+		auto dir = normalize(center - eye);
+		auto lft = normalize(cross(dir, up));
+		auto rup = cross(lft, dir);
+
+		mat<float, 4, 4> ret(1.f);
+		ret.col(0) = { lft.x(), rup.x(), -dir.x(), 0.f };
+		ret.col(1) = { lft.y(), rup.y(), -dir.y(), 0.f };
+		ret.col(2) = { lft.z(), rup.z(), -dir.z(), 0.f };
+		ret.col(3) = { -dot(lft, eye), -dot(rup, eye), dot(dir, eye), 1.f };
+		return ret;
+	}
+
+	/**
+	 * Creates a right-handed perspective matrix, with the near and far clips at -1 and +1, respectively.
+	 * @param fov The FOV in radians
+	 */
+	[[nodiscard]] auto perspectiveRH(float fov, float ratio, float zNear, float zFar) noexcept {
+		mat<float, 4, 4> ret(0.f);
+		auto tanHalfFov = std::tanf(fov / 2.f);
+		ret.col(0).x() = 1.f / (ratio * tanHalfFov);
+		ret.col(1).y() = 1.f / tanHalfFov;
+		ret.col(2).z() = -(zFar + zNear) / (zFar - zNear);
+		ret.col(2).w() = -1.f;
+		ret.col(3).z() = -(2.f * zFar * zNear) / (zFar - zNear);
+		return ret;
+	}
+} // namespace fastgltf::math
 
 constexpr std::string_view vertexShaderSource = R"(
     #version 460 core
@@ -156,8 +179,8 @@ struct IndirectDrawCommand {
 };
 
 struct Vertex {
-	glm::vec3 position;
-	glm::vec2 uv;
+	fastgltf::math::fvec3 position;
+	fastgltf::math::fvec2 uv;
 };
 
 struct Primitive {
@@ -188,11 +211,11 @@ enum MaterialUniformFlags : std::uint32_t {
 };
 
 struct MaterialUniforms {
-    glm::fvec4 baseColorFactor;
-    float alphaCutoff;
-	std::uint32_t flags;
+    fastgltf::math::fvec4 baseColorFactor;
+    float alphaCutoff = 0.f;
+	std::uint32_t flags = 0;
 
-	glm::vec2 padding;
+	fastgltf::math::fvec2 padding;
 };
 
 struct Viewer {
@@ -201,7 +224,7 @@ struct Viewer {
     std::vector<GLuint> bufferAllocations;
     std::vector<Mesh> meshes;
     std::vector<Texture> textures;
-	std::vector<glm::mat4> cameras;
+	std::vector<fastgltf::math::fmat4x4> cameras;
 
     std::vector<MaterialUniforms> materials;
     std::vector<GLuint> materialBuffers;
@@ -210,20 +233,20 @@ struct Viewer {
 	GLint uvScaleUniform = GL_NONE;
 	GLint uvRotationUniform = GL_NONE;
 
-	glm::ivec2 windowDimensions = glm::ivec2(0);
-    glm::mat4 viewMatrix = glm::mat4(1.0f);
-    glm::mat4 projectionMatrix = glm::mat4(1.0f);
+	fastgltf::math::ivec2 windowDimensions = fastgltf::math::ivec2(0);
+    fastgltf::math::fmat4x4 viewMatrix = fastgltf::math::fmat4x4(1.0f);
+    fastgltf::math::fmat4x4 projectionMatrix = fastgltf::math::fmat4x4(1.0f);
     GLint viewProjectionMatrixUniform = GL_NONE;
     GLint modelMatrixUniform = GL_NONE;
 
     float lastFrame = 0.0f;
     float deltaTime = 0.0f;
-    glm::vec3 accelerationVector = glm::vec3(0.0f);
-    glm::vec3 velocity = glm::vec3(0.0f);
-    glm::vec3 position = glm::vec3(0.0f, 0.0f, 0.0f);
+    fastgltf::math::fvec3 accelerationVector = fastgltf::math::fvec3(0.0f);
+    fastgltf::math::fvec3 velocity = fastgltf::math::fvec3(0.0f);
+    fastgltf::math::fvec3 position = fastgltf::math::fvec3(0.0f, 0.0f, 0.0f);
 
-    glm::dvec2 lastCursorPosition = glm::dvec2(0.0f);
-    glm::vec3 direction = glm::vec3(0.0f, 0.0f, -1.0f);
+    fastgltf::math::dvec2 lastCursorPosition = fastgltf::math::dvec2(0.0f);
+    fastgltf::math::fvec3 direction = fastgltf::math::fvec3(0.0f, 0.0f, -1.0f);
     float yaw = -90.0f;
     float pitch = 0.0f;
     bool firstMouse = true;
@@ -234,8 +257,8 @@ struct Viewer {
 };
 
 void updateCameraMatrix(Viewer* viewer) {
-    glm::mat4 viewProjection = viewer->projectionMatrix * viewer->viewMatrix;
-    glUniformMatrix4fv(viewer->viewProjectionMatrixUniform, 1, GL_FALSE, glm::value_ptr(viewProjection));
+    auto viewProjection = viewer->projectionMatrix * viewer->viewMatrix;
+    glUniformMatrix4fv(viewer->viewProjectionMatrixUniform, 1, GL_FALSE, viewProjection.data());
 }
 
 void windowSizeCallback(GLFWwindow* window, int width, int height) {
@@ -262,25 +285,25 @@ void cursorCallback(GLFWwindow* window, double xpos, double ypos) {
         viewer->firstMouse = false;
     }
 
-    auto offset = glm::vec2(xpos - viewer->lastCursorPosition.x, viewer->lastCursorPosition.y - ypos);
+    auto offset = fastgltf::math::fvec2(xpos - viewer->lastCursorPosition.x(), viewer->lastCursorPosition.y() - ypos);
     viewer->lastCursorPosition = { xpos, ypos };
     offset *= 0.1f;
 
-    viewer->yaw   += offset.x;
-    viewer->pitch += offset.y;
-    viewer->pitch = glm::clamp(viewer->pitch, -89.0f, 89.0f);
+    viewer->yaw   += offset.x();
+    viewer->pitch += offset.y();
+    viewer->pitch = fastgltf::math::clamp(viewer->pitch, -89.0f, 89.0f);
 
     auto& direction = viewer->direction;
-    direction.x = cos(glm::radians(viewer->yaw)) * cos(glm::radians(viewer->pitch));
-    direction.y = sin(glm::radians(viewer->pitch));
-    direction.z = sin(glm::radians(viewer->yaw)) * cos(glm::radians(viewer->pitch));
-    direction = glm::normalize(direction);
+    direction.x() = cos(fastgltf::math::radians(viewer->yaw)) * cos(fastgltf::math::radians(viewer->pitch));
+    direction.y() = sin(fastgltf::math::radians(viewer->pitch));
+    direction.z() = sin(fastgltf::math::radians(viewer->yaw)) * cos(fastgltf::math::radians(viewer->pitch));
+    direction = fastgltf::math::normalize(direction);
 }
 
 void keyCallback(GLFWwindow* window, int key, int scancode, int action, int mods) {
     void* ptr = glfwGetWindowUserPointer(window);
     auto* viewer = static_cast<Viewer*>(ptr);
-    static constexpr auto cameraUp = glm::vec3(0.0f, 1.0f, 0.0f);
+    static constexpr auto cameraUp = fastgltf::math::fvec3(0.0f, 1.0f, 0.0f);
 
     auto& acceleration = viewer->accelerationVector;
     switch (key) {
@@ -291,46 +314,27 @@ void keyCallback(GLFWwindow* window, int key, int scancode, int action, int mods
             acceleration -= viewer->direction;
             break;
         case GLFW_KEY_D:
-            acceleration += glm::normalize(glm::cross(viewer->direction, cameraUp));
+            acceleration += fastgltf::math::normalize(fastgltf::math::cross(viewer->direction, cameraUp));
             break;
         case GLFW_KEY_A:
-            acceleration -= glm::normalize(glm::cross(viewer->direction, cameraUp));
+            acceleration -= fastgltf::math::normalize(fastgltf::math::cross(viewer->direction, cameraUp));
             break;
         default:
             break;
     }
 }
 
-glm::mat4 getTransformMatrix(const fastgltf::Node& node, glm::mat4x4& base) {
-    /** Both a matrix and TRS values are not allowed
-     * to exist at the same time according to the spec */
-    if (const auto* pMatrix = std::get_if<fastgltf::Node::TransformMatrix>(&node.transform)) {
-        return base * glm::mat4x4(glm::make_mat4x4(pMatrix->data()));
-    }
-
-	if (const auto* pTransform = std::get_if<fastgltf::TRS>(&node.transform)) {
-		// Warning: The quaternion to mat4x4 conversion here is not correct with all versions of glm.
-		// glTF provides the quaternion as (x, y, z, w), which is the same layout glm used up to version 0.9.9.8.
-		// However, with commit 59ddeb7 (May 2021) the default order was changed to (w, x, y, z).
-		// You could either define GLM_FORCE_QUAT_DATA_XYZW to return to the old layout,
-		// or you could use the recently added static factory constructor glm::quat::wxyz(w, x, y, z),
-		// which guarantees the parameter order.
-        return base
-            * glm::translate(glm::mat4(1.0f), glm::make_vec3(pTransform->translation.data()))
-            * glm::toMat4(glm::make_quat(pTransform->rotation.data()))
-            * glm::scale(glm::mat4(1.0f), glm::make_vec3(pTransform->scale.data()));
-    }
-
-	return base;
-}
-
-bool loadGltf(Viewer* viewer, std::string_view cPath) {
-	if (!std::filesystem::exists(cPath)) {
-		std::cout << "Failed to find " << cPath << '\n';
+bool loadGltf(Viewer* viewer, std::filesystem::path path) {
+	if (!std::filesystem::exists(path)) {
+		std::cout << "Failed to find " << path << '\n';
 		return false;
 	}
 
-    std::cout << "Loading " << cPath << '\n';
+	if constexpr (std::is_same_v<std::filesystem::path::value_type, wchar_t>) {
+		std::wcout << "Loading " << path << '\n';
+	} else {
+		std::cout << "Loading " << path << '\n';
+	}
 
     // Parse the glTF file and get the constructed asset
     {
@@ -341,8 +345,6 @@ bool loadGltf(Viewer* viewer, std::string_view cPath) {
 
         fastgltf::Parser parser(supportedExtensions);
 
-        auto path = std::filesystem::path{cPath};
-
         constexpr auto gltfOptions =
             fastgltf::Options::DontRequireValidAssetMember |
             fastgltf::Options::AllowDouble |
@@ -351,10 +353,13 @@ bool loadGltf(Viewer* viewer, std::string_view cPath) {
             fastgltf::Options::LoadExternalImages |
 			fastgltf::Options::GenerateMeshIndices;
 
-        fastgltf::GltfDataBuffer data;
-        data.loadFromFile(path);
+		auto gltfFile = fastgltf::MappedGltfFile::FromPath(path);
+		if (!bool(gltfFile)) {
+			std::cerr << "Failed to open glTF file: " << fastgltf::getErrorMessage(gltfFile.error()) << '\n';
+			return false;
+		}
 
-        auto asset = parser.loadGltf(&data, path.parent_path(), gltfOptions);
+        auto asset = parser.loadGltf(gltfFile.get(), path.parent_path(), gltfOptions);
         if (asset.error() != fastgltf::Error::None) {
             std::cerr << "Failed to load glTF: " << fastgltf::getErrorMessage(asset.error()) << '\n';
             return false;
@@ -410,7 +415,7 @@ bool loadMesh(Viewer* viewer, fastgltf::Mesh& mesh) {
 
         {
             // Position
-            auto& positionAccessor = asset.accessors[positionIt->second];
+            auto& positionAccessor = asset.accessors[positionIt->accessorIndex];
             if (!positionAccessor.bufferViewIndex.has_value())
                 continue;
 
@@ -418,9 +423,9 @@ bool loadMesh(Viewer* viewer, fastgltf::Mesh& mesh) {
 			glCreateBuffers(1, &primitive.vertexBuffer);
 			glNamedBufferData(primitive.vertexBuffer, positionAccessor.count * sizeof(Vertex), nullptr, GL_STATIC_DRAW);
 			auto* vertices = static_cast<Vertex*>(glMapNamedBuffer(primitive.vertexBuffer, GL_WRITE_ONLY));
-			fastgltf::iterateAccessorWithIndex<glm::vec3>(asset, positionAccessor, [&](glm::vec3 pos, std::size_t idx) {
-				vertices[idx].position = pos;
-				vertices[idx].uv = glm::vec2();
+			fastgltf::iterateAccessorWithIndex<fastgltf::math::fvec3>(asset, positionAccessor, [&](fastgltf::math::fvec3 pos, std::size_t idx) {
+				vertices[idx].position = fastgltf::math::fvec3(pos.x(), pos.y(), pos.z());
+				vertices[idx].uv = fastgltf::math::fvec2();
 			});
 			glUnmapNamedBuffer(primitive.vertexBuffer);
 
@@ -437,13 +442,13 @@ bool loadMesh(Viewer* viewer, fastgltf::Mesh& mesh) {
 		auto texcoordAttribute = std::string("TEXCOORD_") + std::to_string(baseColorTexcoordIndex);
         if (const auto* texcoord = it->findAttribute(texcoordAttribute); texcoord != it->attributes.end()) {
             // Tex coord
-			auto& texCoordAccessor = asset.accessors[texcoord->second];
+			auto& texCoordAccessor = asset.accessors[texcoord->accessorIndex];
             if (!texCoordAccessor.bufferViewIndex.has_value())
                 continue;
 
 			auto* vertices = static_cast<Vertex*>(glMapNamedBuffer(primitive.vertexBuffer, GL_WRITE_ONLY));
-			fastgltf::iterateAccessorWithIndex<glm::vec2>(asset, texCoordAccessor, [&](glm::vec2 uv, std::size_t idx) {
-				vertices[idx].uv = uv;
+			fastgltf::iterateAccessorWithIndex<fastgltf::math::fvec2>(asset, texCoordAccessor, [&](fastgltf::math::fvec2 uv, std::size_t idx) {
+				vertices[idx].uv = fastgltf::math::fvec2(uv.x(), uv.y());
 			});
 			glUnmapNamedBuffer(primitive.vertexBuffer);
 
@@ -469,14 +474,26 @@ bool loadMesh(Viewer* viewer, fastgltf::Mesh& mesh) {
             return false;
         draw.count = static_cast<std::uint32_t>(indexAccessor.count);
 
-		// Create the index buffer and copy 32-bit indices into it.
+		// Create the index buffer and copy the indices into it.
 		glCreateBuffers(1, &primitive.indexBuffer);
-		glNamedBufferData(primitive.indexBuffer, static_cast<GLsizeiptr>(indexAccessor.count * sizeof(std::uint32_t)), nullptr, GL_STATIC_DRAW);
-		auto* indices = static_cast<std::uint32_t*>(glMapNamedBuffer(primitive.indexBuffer, GL_WRITE_ONLY));
-		fastgltf::copyFromAccessor<std::uint32_t>(asset, indexAccessor, indices);
-		glUnmapNamedBuffer(primitive.indexBuffer);
+		if (indexAccessor.componentType == fastgltf::ComponentType::UnsignedByte || indexAccessor.componentType == fastgltf::ComponentType::UnsignedShort) {
+        	primitive.indexType = GL_UNSIGNED_SHORT;
+			glNamedBufferData(primitive.indexBuffer,
+							  static_cast<GLsizeiptr>(indexAccessor.count * sizeof(std::uint16_t)), nullptr,
+							  GL_STATIC_DRAW);
+			auto* indices = static_cast<std::uint16_t*>(glMapNamedBuffer(primitive.indexBuffer, GL_WRITE_ONLY));
+			fastgltf::copyFromAccessor<std::uint16_t>(asset, indexAccessor, indices);
+			glUnmapNamedBuffer(primitive.indexBuffer);
+		} else {
+        	primitive.indexType = GL_UNSIGNED_INT;
+			glNamedBufferData(primitive.indexBuffer,
+							  static_cast<GLsizeiptr>(indexAccessor.count * sizeof(std::uint32_t)), nullptr,
+							  GL_STATIC_DRAW);
+			auto* indices = static_cast<std::uint32_t*>(glMapNamedBuffer(primitive.indexBuffer, GL_WRITE_ONLY));
+			fastgltf::copyFromAccessor<std::uint32_t>(asset, indexAccessor, indices);
+			glUnmapNamedBuffer(primitive.indexBuffer);
+		}
 
-        primitive.indexType = GL_UNSIGNED_INT;
         glVertexArrayElementBuffer(vao, primitive.indexBuffer);
     }
 
@@ -512,7 +529,7 @@ bool loadImage(Viewer* viewer, fastgltf::Image& image) {
         },
         [&](fastgltf::sources::Array& vector) {
             int width, height, nrChannels;
-            unsigned char *data = stbi_load_from_memory(vector.bytes.data(), static_cast<int>(vector.bytes.size()), &width, &height, &nrChannels, 4);
+            unsigned char *data = stbi_load_from_memory(reinterpret_cast<const stbi_uc*>(vector.bytes.data()), static_cast<int>(vector.bytes.size()), &width, &height, &nrChannels, 4);
             glTextureStorage2D(texture, getLevelCount(width, height), GL_RGBA8, width, height);
             glTextureSubImage2D(texture, 0, 0, 0, width, height, GL_RGBA, GL_UNSIGNED_BYTE, data);
             stbi_image_free(data);
@@ -528,7 +545,8 @@ bool loadImage(Viewer* viewer, fastgltf::Image& image) {
                 [](auto& arg) {},
                 [&](fastgltf::sources::Array& vector) {
                     int width, height, nrChannels;
-                    unsigned char* data = stbi_load_from_memory(vector.bytes.data() + bufferView.byteOffset, static_cast<int>(bufferView.byteLength), &width, &height, &nrChannels, 4);
+					unsigned char* data = stbi_load_from_memory(reinterpret_cast<const stbi_uc*>(vector.bytes.data() + bufferView.byteOffset),
+					                                            static_cast<int>(bufferView.byteLength), &width, &height, &nrChannels, 4);
                     glTextureStorage2D(texture, getLevelCount(width, height), GL_RGBA8, width, height);
                     glTextureSubImage2D(texture, 0, 0, 0, width, height, GL_RGBA, GL_UNSIGNED_BYTE, data);
                     stbi_image_free(data);
@@ -547,7 +565,7 @@ bool loadMaterial(Viewer* viewer, fastgltf::Material& material) {
     MaterialUniforms uniforms = {};
     uniforms.alphaCutoff = material.alphaCutoff;
 
-    uniforms.baseColorFactor = glm::make_vec4(material.pbrData.baseColorFactor.data());
+    uniforms.baseColorFactor = material.pbrData.baseColorFactor;
     if (material.pbrData.baseColorTexture.has_value()) {
         uniforms.flags |= MaterialUniformFlags::HasBaseColorTexture;
     }
@@ -561,7 +579,7 @@ bool loadCamera(Viewer* viewer, fastgltf::Camera& camera) {
 	// https://registry.khronos.org/glTF/specs/2.0/glTF-2.0.html#projection-matrices
 	std::visit(fastgltf::visitor {
 		[&](fastgltf::Camera::Perspective& perspective) {
-			glm::mat4x4 mat(0.0f);
+			fastgltf::math::fmat4x4 mat(0.0f);
 
 			assert(viewer->windowDimensions[0] != 0 && viewer->windowDimensions[1] != 0);
 			auto aspectRatio = perspective.aspectRatio.value_or(
@@ -582,7 +600,7 @@ bool loadCamera(Viewer* viewer, fastgltf::Camera& camera) {
 			viewer->cameras.emplace_back(mat);
 		},
 		[&](fastgltf::Camera::Orthographic& orthographic) {
-			glm::mat4x4 mat(1.0f);
+			fastgltf::math::fmat4x4 mat(1.0f);
 			mat[0][0] = 1.f / orthographic.xmag;
 			mat[1][1] = 1.f / orthographic.ymag;
 			mat[2][2] = 2.f / (orthographic.znear - orthographic.zfar);
@@ -593,7 +611,7 @@ bool loadCamera(Viewer* viewer, fastgltf::Camera& camera) {
 	return true;
 }
 
-void drawMesh(Viewer* viewer, std::size_t meshIndex, glm::mat4 matrix) {
+void drawMesh(Viewer* viewer, std::size_t meshIndex, fastgltf::math::fmat4x4 matrix) {
     auto& mesh = viewer->meshes[meshIndex];
 
     glBindBuffer(GL_DRAW_INDIRECT_BUFFER, mesh.drawsBuffer);
@@ -636,19 +654,6 @@ void drawMesh(Viewer* viewer, std::size_t meshIndex, glm::mat4 matrix) {
     }
 }
 
-void drawNode(Viewer* viewer, std::size_t nodeIndex, glm::mat4 matrix) {
-    auto& node = viewer->asset.nodes[nodeIndex];
-    matrix = getTransformMatrix(node, matrix);
-
-    if (node.meshIndex.has_value()) {
-        drawMesh(viewer, node.meshIndex.value(), matrix);
-    }
-
-    for (auto& child : node.children) {
-        drawNode(viewer, child, matrix);
-    }
-}
-
 void updateCameraNodes(Viewer* viewer, std::vector<fastgltf::Node*>& cameraNodes, std::size_t nodeIndex) {
 	// This function recursively traverses the node hierarchy starting with the node at nodeIndex
 	// to find any nodes holding cameras.
@@ -667,25 +672,22 @@ void updateCameraNodes(Viewer* viewer, std::vector<fastgltf::Node*>& cameraNodes
 	}
 }
 
-void getCameraViewMatrix(Viewer* viewer, std::vector<fastgltf::Node*>& cameraNodes, std::size_t nodeIndex, glm::mat4 matrix) {
-	auto& node = viewer->asset.nodes[nodeIndex];
-	matrix = getTransformMatrix(node, matrix);
-
-	if (node.cameraIndex.has_value() && &node == cameraNodes[*viewer->cameraIndex]) {
-		viewer->viewMatrix = matrix;
+#ifdef _MSC_VER
+int wmain(int argc, wchar_t* argv[]) {
+	if (argc < 2) {
+		std::cerr << "No gltf file specified." << '\n';
+		return -1;
 	}
-
-	for (auto& child : node.children) {
-		getCameraViewMatrix(viewer, cameraNodes, child, matrix);
-	}
-}
-
+	auto gltfFile = std::wstring_view { argv[1] };
+#else
 int main(int argc, char* argv[]) {
     if (argc < 2) {
         std::cerr << "No gltf file specified." << '\n';
         return -1;
     }
     auto gltfFile = std::string_view { argv[1] };
+#endif
+
     Viewer viewer;
 
     if (glfwInit() != GLFW_TRUE) {
@@ -785,7 +787,7 @@ int main(int argc, char* argv[]) {
 
 	// Add a default material
 	auto& defaultMaterial = viewer.materials.emplace_back();
-	defaultMaterial.baseColorFactor = glm::vec4(1.0f);
+	defaultMaterial.baseColorFactor = fastgltf::math::fvec4(1.0f);
 	defaultMaterial.alphaCutoff = 0.0f;
 	defaultMaterial.flags = 0;
 
@@ -847,11 +849,11 @@ int main(int argc, char* argv[]) {
 	}
 
 	// Set the initial direction (incl. pitch and yaw) and position of the camera.
-	viewer.position = glm::vec3(2.f, 2.f, 2.f);
+	viewer.position = fastgltf::math::fvec3(2.f, 2.f, 2.f);
 	viewer.direction = -viewer.position;
 	{
-		auto len = std::sqrt(std::pow(viewer.direction.x, 2) + std::pow(viewer.direction.z, 2));
-		viewer.pitch = glm::degrees(std::tan(viewer.direction.y / len));
+		auto len = std::sqrtf(std::powf(viewer.direction.x(), 2) + std::powf(viewer.direction.z(), 2));
+		viewer.pitch = fastgltf::math::degrees(std::tanf(viewer.direction.y() / len));
 		viewer.yaw = -135.f;
 	}
 
@@ -862,7 +864,7 @@ int main(int argc, char* argv[]) {
         viewer.lastFrame = currentFrame;
 
         // Reset the acceleration
-        viewer.accelerationVector = glm::vec3(0.0f);
+        viewer.accelerationVector = fastgltf::math::fvec3(0.0f);
 
         // Updates the acceleration vector and direction vectors.
         glfwPollEvents();
@@ -942,37 +944,40 @@ int main(int argc, char* argv[]) {
         glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
 		if (!asset.scenes.empty() && sceneIndex < asset.scenes.size()) {
-			auto& scene = asset.scenes[sceneIndex];
-
 			// Update the camera view and projection matrices
 			if (viewer.cameraIndex.has_value()) {
-				for (auto& sceneNode: scene.nodeIndices) {
-					getCameraViewMatrix(&viewer, cameraNodes, sceneNode, glm::mat4(1.0f));
-				}
+				fastgltf::iterateSceneNodes(asset, sceneIndex, fastgltf::math::fmat4x4(),
+											[&](fastgltf::Node& node, fastgltf::math::fmat4x4 matrix) {
+					if (node.cameraIndex.has_value() && &node == cameraNodes[*viewer.cameraIndex]) {
+						viewer.viewMatrix = matrix;
+					}
+				});
 
-				viewer.viewMatrix = glm::affineInverse(viewer.viewMatrix);
+				viewer.viewMatrix = fastgltf::math::affineInverse(viewer.viewMatrix);
 				viewer.projectionMatrix = viewer.cameras[viewer.cameraIndex.value()];
 			} else {
 				// Factor the deltaTime into the amount of acceleration
 				viewer.velocity += (viewer.accelerationVector * 50.0f) * viewer.deltaTime;
 				// Lerp the velocity to 0, adding deceleration.
-				viewer.velocity = viewer.velocity + (2.0f * viewer.deltaTime) * (glm::vec3(0.0f) - viewer.velocity);
+				viewer.velocity = viewer.velocity + (-viewer.velocity) * (2.0f * viewer.deltaTime);
 				// Add the velocity into the position
 				viewer.position += viewer.velocity * viewer.deltaTime;
-				viewer.viewMatrix = glm::lookAt(viewer.position, viewer.position + viewer.direction,
-												glm::vec3(0.0f, 1.0f, 0.0f));
+				viewer.viewMatrix = fastgltf::math::lookAtRH(viewer.position, viewer.position + viewer.direction,
+												fastgltf::math::fvec3(0.0f, 1.0f, 0.0f));
 
 				auto aspectRatio = static_cast<float>(viewer.windowDimensions[0]) / static_cast<float>(viewer.windowDimensions[1]);
-				viewer.projectionMatrix = glm::perspective(glm::radians(75.0f),
-														   aspectRatio,
-															0.01f, 1000.0f);
+				viewer.projectionMatrix = fastgltf::math::perspectiveRH(
+					fastgltf::math::radians(75.0f), aspectRatio, 0.01f, 1000.0f);
 			}
 
 			updateCameraMatrix(&viewer);
 
-			for (auto& node: scene.nodeIndices) {
-				drawNode(&viewer, node, glm::mat4(1.0f));
-			}
+			fastgltf::iterateSceneNodes(asset, sceneIndex, fastgltf::math::fmat4x4(),
+										[&](fastgltf::Node& node, fastgltf::math::fmat4x4 matrix) {
+				if (node.meshIndex.has_value()) {
+					drawMesh(&viewer, *node.meshIndex, matrix);
+				}
+			});
 		}
 
 		// Render ImGui
